'
' ####################
' #####  PROLOG  #####
' ####################
'
' An example program that reads words from a text-file (a
' book or something like that) and uses those as keys in a hash table
' (the actual data stored is unimportant). The words are case
' sensitive.
'
' The program then opens another text-file and tries to match
' the words in that with the words stored in the table.
' See dict_example.c
'
VERSION "0.0001"
CONSOLE
'
	IMPORT "xst"				' Standard library : required by most programs
	IMPORT "hash_table"
	IMPORT "msvcrt"
	IMPORT "xsx"				' Extended standard library
'	IMPORT "xio"				' Console input/ouput library

'	IMPORT "xst_s.lib"
' IMPORT "xsx_s.lib"
' IMPORT "xio_s.lib"

' IMPORT "gdi32"			' gdi32.dll
'	IMPORT "user32"		  ' user32.dll
'	IMPORT "kernel32"	  ' kernel32.dll
'	IMPORT "shell32"		' shell32.dll

'
DECLARE FUNCTION Entry ()

$$DELIMS = " \".,;:?!-\'/*()=+&%[]#$\n\r\t"
'
'
' ######################
' #####  Entry ()  #####
' ######################
'
FUNCTION Entry ()

ght_iterator_t iterator

  ' Create a new hash table 
	p_table = ght_create (1000) 
  IFZ p_table THEN RETURN 1

  ght_set_rehash (p_table, 1)

	' load the dictionary file
	XstLoadString ("dictionary.x", @p_buf$)
	
	' For each word in the dictionary file, insert it into the hash table.
	
	' Returns first token
  p_tok = strtok (&p_buf$, &$$DELIMS)
 
  ' Keep parsing tokens while one of the delimiters present in p_buf$
	i_cnt = 0
	i_found = 0
  DO WHILE (p_tok != NULL)
'    printf(&" % s\n", p_tok)
		p_data = Xmalloc(SIZE(XLONG))
		IFZ p_data THEN
			PRINT "malloc error"
			RETURN 1
		END IF
		
		INC i_cnt
		XLONGAT(p_data) = i_cnt
		
		IF (ght_insert (p_table, p_data, strlen(p_tok), p_tok) < 0) THEN 
'			Xfree(p_data) ' Could not insert the item (already exists), free it. 
    ELSE 
			INC i_found
		END IF 
		p_tok = strtok (NULL, &$$DELIMS)
		
  LOOP
	
	printf (&"Done reading %d unique words from the wordlist.\r\nTotal number of words is %d.\r\n", i_found, i_cnt)
	
	' load the comparison file xconsole.xxx
	XstGetEnvironmentVariable ("XBLDIR", @path$)
	path$ = path$ + "/demos/console/contest/contest.x"
	PRINT path$
	XstLoadString (path$, @p_buf$)
	
	' For each word in the text file, check if it exists in the hash table.
  p_tok = strtok (&p_buf$, &$$DELIMS)
  i_cnt = 0
  i_found = 0
  DO WHILE (p_tok)
'		printf(&" % s\n", p_tok)
    IF (ght_get (p_table, strlen(p_tok), p_tok)) THEN INC i_found
		INC i_cnt
    p_tok = strtok(NULL, &$$DELIMS)
  LOOP

  printf (&"Found %d words out of %d words\r\n", i_found, i_cnt)
	
	' Free the entry data in the table
	p_e = ght_first (p_table, &iterator, &p_key) 

  DO WHILE p_e
'		PRINT XLONGAT(p_e), " : "; CSTRING$(p_key) 
		IF p_e THEN Xfree (p_e)
		p_e = ght_next (p_table, &iterator, &p_key)
  LOOP

  ' Free the table 
  ght_finalize (p_table)
	

  a$ = INLINE$ ("Press Enter to quit >")

END FUNCTION
END PROGRAM